/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate Data files.

  Copyright (C) 2003-2005 Uwe Schulzweida, schulzweida@dkrz.de
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef _CDO_H
#define _CDO_H

#include <stdio.h>
#include "dmemory.h"

#define  TRUE   1
#define  FALSE  0

#undef   MIN
#define  MIN(a,b)  ((a) < (b) ? (a) : (b))
#undef   MAX
#define  MAX(a,b)  ((a) > (b) ? (a) : (b))
#undef   NINT
#define  NINT(x)   ((x) < 0 ? (int)((x)-0.5) : (int)((x)+0.5))

#define  UNCHANGED_RECORD  (processSelf() == 0 && *cdoStreamName(0) != '-' && cdoRegulargrid == FALSE)


typedef struct {
  int      argc;
  char   **argv;
}
ARGUMENT;


extern int cdoDefaultFileType;
extern int cdoDefaultDataType;
extern int cdoDefaultByteorder;
extern int cdoDefaultTableID;
extern int cdoDefaultInstID;

extern int cdoRegulargrid;
extern int cdoBenchmark;
extern int cdoVerbose;
extern int cdoDebug;

void    cdiError(int cdiErrno, const char *fmt, ...);
void    cdoAbort(const char *fmt, ...);
void    cdoWarning(const char *fmt, ...);
void    cdoPrint(const char *fmt, ...);

void    operatorInputArg(const char *enter);
int     operatorArgc(void);
char  **operatorArgv(void);

int     args2intarr(int argc, char **argv, int intarr[], int maxvals);

const char *cdoStreamName(int cnt);

void    cdoInitialize(void *argument);
void    cdoFinish(void);

int     cdoStreamCnt(void);
int     cdoOperatorID(int noperators, const char *operators[]);

int     cdoFiletype(void);

void    cdoInqHistory(int fileID);
void    cdoDefHistory(int fileID, char *histstring);

int     cdoDefineGrid(const char *gridfile);

void    vlistCompare(int vlistID1, int vlistID2, int function);

void gridArea(int gridID, double *area);
void gaussaw(double pa[], double pw[], int nlat);
void genXbounds(int xsize, int ysize, double *grid_center_lon, double *grid_corner_lon);
void genYbounds(int xsize, int ysize, double *grid_center_lat, double *grid_corner_lat);
void writeNCgrid(const char *gridfile, int gridID, int *imask);
void defineZaxis(const char *zaxisarg);
void cdiDefTableID(int tableID);
void gridGenXvals(int xsize, double xfirst, double xlast, double xinc, double *xvals);
void gridGenYvals(int gridtype, int ysize, double yfirst, double ylast, double yinc, double *yvals);

int gridFromName(const char *gridname);

#endif  /* _CDO_H */
