/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate Data files.

  Copyright (C) 2003-2005 Uwe Schulzweida, schulzweida@dkrz.de
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef _FIELD_H
#define _FIELD_H


#define  ADD(x,y)  ( (x)==missval1 || (y)==missval2 ? missval1 : (x)+(y) )
#define  SUB(x,y)  ( (x)==missval1 || (y)==missval2 ? missval1 : (x)-(y) )
#define  MUL(x,y)  ( (x)==missval1 || (y)==missval2 ? missval1 : (x)*(y) )
#define  DIV(x,y)  ( (x)==missval1 || (y)==missval2 || (y)==0 ? missval1 : (x)/(y) )
#define  ROOT(x)   ( (x)==missval1 || (x)<0 ? missval1 : sqrt(x) )


typedef struct {
  int      grid;
  int      zaxis;
  int      nsamp;
  int      nmiss;
  double   missval;
  double  *weight;
  double  *ptr;
}
FIELD;

/* field.c */

void fldfun(FIELD field1, FIELD *field2, int function);
void fldmin(FIELD field1, FIELD *field2);
void fldmax(FIELD field1, FIELD *field2);
void fldsum(FIELD field1, FIELD *field2);
void fldavg(FIELD field1, FIELD *field2);
void fldmean(FIELD field1, FIELD *field2);
void fldstd(FIELD field1, FIELD *field2);
void fldvar(FIELD field1, FIELD *field2);

double minval(FIELD field);
double maxval(FIELD field);

/* fieldzon.c */

void zonfun(FIELD field1, FIELD *field2, int function);
void zonmin(FIELD field1, FIELD *field2);
void zonmax(FIELD field1, FIELD *field2);
void zonsum(FIELD field1, FIELD *field2);
void zonavg(FIELD field1, FIELD *field2);
void zonmean(FIELD field1, FIELD *field2);
void zonstd(FIELD field1, FIELD *field2);
void zonvar(FIELD field1, FIELD *field2);

/* fieldmer.c */

void merfun(FIELD field1, FIELD *field2, int function);
void mermin(FIELD field1, FIELD *field2);
void mermax(FIELD field1, FIELD *field2);
void mersum(FIELD field1, FIELD *field2);
void meravg(FIELD field1, FIELD *field2);
void mermean(FIELD field1, FIELD *field2);
void merstd(FIELD field1, FIELD *field2);
void mervar(FIELD field1, FIELD *field2);

void fldrms(FIELD field1, FIELD field2, FIELD *field3);

void varrms(FIELD field1, FIELD field2, FIELD *field3);

/* fieldc.c */

void farcfun(FIELD *field, double rconst, int function);

void farcmul(FIELD *field, double rconst);
void farcdiv(FIELD *field, double rconst);
void farcadd(FIELD *field, double rconst);
void farcsub(FIELD *field, double rconst);

void farinv(FIELD *field);

/* field2.c */

void farfun(FIELD *field1, FIELD field2, int function);

void faradd(FIELD *field1, FIELD field2);
void farsum(FIELD *field1, FIELD field2);
void farsumq(FIELD *field1, FIELD field2);
void farsub(FIELD *field1, FIELD field2);
void farmul(FIELD *field1, FIELD field2);
void fardiv(FIELD *field1, FIELD field2);
void farmin(FIELD *field1, FIELD field2);
void farmax(FIELD *field1, FIELD field2);
void farstd(FIELD *field1, FIELD field2, FIELD field3);
void farcstd(FIELD *field1, FIELD field2, double rconst1);
void farmoq(FIELD *field1, FIELD field2);

#endif  /* _FIELD_H */
