/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate Data files.

  Copyright (C) 2003-2005 Uwe Schulzweida, schulzweida@dkrz.de
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "cdo_int.h"

static void split_intstring(const char *intstr, int *first, int *last, int *inc)
{
  int i, start;
  int istrlen;

  istrlen = strlen(intstr);
  *first = atoi(intstr);
  *last  = *first;
  *inc   = 1;

  start = 1;
  for ( i = start; i < istrlen; i++ )
    {
      if ( ! isdigit(intstr[i]) )
	{
	  if ( intstr[i] == '/' )
	    *last = atoi(intstr+i+1);
	  else
	    fprintf(stderr, "Syntax error in >%.*s<! Character %c not allowed.\n",
		    istrlen, intstr, intstr[i]);
	  break;
	}
    }

  if ( i < istrlen )
    {
      start = i+1;
      for ( i = start; i < istrlen; i++ )
	{
	  if ( ! isdigit(intstr[i]) )
	    {
	      if ( intstr[i] == '/' )
		*inc = atoi(intstr+i+1);
	      else
		fprintf(stderr, "Syntax error in >%.*s<! Character %c not allowed.\n",
			istrlen, intstr, intstr[i]);
	      break;
	    }
	}
    }
}

int args2intarr(int argc, char **argv, int intarr[], int maxvals)
{
  int nint = 0;
  int ival;
  int first, last, inc;
  int iarg;

  for ( iarg = 0; iarg < argc; iarg++ )
    {
      split_intstring(argv[iarg], &first, &last, &inc);

      for ( ival = first; ival <= last; ival += inc )
	{
	  if ( nint >= maxvals )
	    {
	      fprintf(stderr, "Warning: Too much values specified. Max values is %d!\n",
		      maxvals);
	      break;
	    }
	  intarr[nint++] = ival;
	}

      if ( nint >= maxvals ) break;
    }

  return (nint);
}
