/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate Data files.

  Copyright (C) 2003-2005 Uwe Schulzweida, schulzweida@dkrz.de
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#ifndef _PIPE_H
#define _PIPE_H

#if  defined  (HAVE_CONFIG_H)
#  include "config.h"
#endif

#if  defined  (HAVE_PTHREAD_H)

#include <pthread.h>
#include <pthread_debug.h>

typedef struct {
  int nrecs, EOP;
  int varID, levelID;
  int recIDr, recIDw, tsIDr, tsIDw;
  int hasdata, usedata, pstreamIDin;
  int nmiss;
  double *data;
  pthread_mutex_t *mutex;
  pthread_cond_t *tsDef, *tsInq, *vlistDef, *isclosed;
  pthread_cond_t *recDef, *recInq;
  pthread_cond_t *writeCond, *readCond;
}
PIPE;

void  pipeInit(PIPE *pipe);
void  pipeDebug(int debug);

void  pipeDefVlist(int pstreamID, int vlistID);
int   pipeInqVlist(int pstreamID);

void  pipeDefTimestep(int pstreamID, int tsID);
int   pipeInqTimestep(int pstreamID, int tsID);

void  pipeDefRecord(int pstreamID, int  varID, int  levelID);
int   pipeInqRecord(int pstreamID, int *varID, int *levelID);

void  pipeReadRecord(int pstreamID, double *data, int *nmiss);
void  pipeWriteRecord(int pstreamID, double *data, int nmiss);
void  pipeCopyRecord(int pstreamIDdest, int pstreamIDsrc);

#endif

#endif  /* _PIPE_H */
