/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate Data files.

  Copyright (C) 2003-2005 Uwe Schulzweida, schulzweida@dkrz.de
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#if  defined  (HAVE_CONFIG_H)
#  include "config.h"
#endif

#if  defined  (HAVE_PTHREAD_H)

#include <pthread.h>
#include "errno.h"
#include "error.h"

void Pthread_mutex_lock(pthread_mutex_t *mutex)
{
  static char func[] = "Pthread_mutex_lock";
  int status;

  status = pthread_mutex_lock(mutex);
  if ( status != 0 )
    {
      switch (status)
	{
	case EINVAL:
	  Error(func, "the mutex has not been properly initialized");
	  break;
	case EDEADLK:
	  Error(func, "the  mutex  is  already locked by the calling thread");
	  break;
	default:
	  Error(func, "status %d unknown", status);
	}
    }
}

void Pthread_mutex_unlock(pthread_mutex_t *mutex)
{
  static char func[] = "Pthread_mutex_unlock";
  int status;

  status = pthread_mutex_unlock(mutex);
  if ( status != 0 )
    {
      switch (status)
	{
	case EINVAL:
	  Error(func, "the mutex has not been properly initialized");
	  break;
	case EPERM:
	  Error(func, "the calling thread does not own the mutex");
	  break;
	default:
	  Error(func, "status %d unknown", status);
	}
    }
}

#endif
